Package: conMItion
Type: Package
Title: Conditional Mutual Information Estimation for Multi-Omics Data
Version: 0.4.0
Authors@R: c(
    person(
      "Gaojianyong", "Wang",
      email = "gjywang@gmail.com",
      role = c("aut", "cre")
    )
  )
Description: The biases introduced in association measures, particularly mutual information, 
    are influenced by factors such as tumor purity, mutation burden, and hypermethylation. 
    This package provides the estimation of conditional mutual information (CMI) and its 
    statistical significance with a focus on its application to multi-omics data. Utilizing 
    B-spline functions (inspired by Daub et al. (2004) <doi:10.1186/1471-2105-5-118>), the package offers tools to estimate the association between heterogeneous multi-
    omics data, while removing the effects of confounding factors. This helps to unravel complex
    biological interactions. In addition, it includes methods to evaluate the statistical significance 
    of these associations, providing a robust framework for multi-omics data integration and 
    analysis. This package is ideal for researchers in computational biology, bioinformatics, 
    and systems biology seeking a comprehensive tool for understanding interdependencies in 
    omics data.
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-17 14:34:15 UTC; gwang
Author: Gaojianyong Wang [aut, cre]
Maintainer: Gaojianyong Wang <gjywang@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-17 15:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-06-17 18:49:32 UTC; unix
Archs: conMItion.so.dSYM

# Generated by roxygen2: do not edit by hand

export(CMIBiCondimat2mat)
export(CMIBiCondimat2matAll)
export(CMIBiCondimat2matPermu)
export(CMIBiCondimat2vec)
export(CMIBiCondimat2vecPermu)
export(CMImat2mat)
export(CMImat2matAll)
export(CMImat2matPermu)
export(CMImat2vec)
export(CMImat2vecPermu)
export(CORmat2vecPermu)
export(MImat2mat)
export(MImat2matAll)
export(MImat2matPermu)
export(MImat2vec)
export(MImat2vecPermu)
export(getCMI)
export(getCMIBiCondi)
export(getEntropy)
export(getEntropyBi)
export(getEntropyQuadri)
export(getEntropyTri)
export(getMI)
export(getMIBi)
export(getPValue)
importFrom(stats,cor)
importFrom(stats,runif)
useDynLib(conMItion)

#' @title Calculate Mutual Information Between Two Vectors
#' @description Computes the mutual information (MI) between two numeric vectors
#' using the specified number of bins and spline order.
#'
#' @param x_1 A numeric vector representing the first variable.
#' @param x_2 A numeric vector representing the second variable. Must be the same length as `x_1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric value representing the mutual information (MI).
#'
#' @examples
#' x_1 <- rnorm(100)
#' x_2 <- rnorm(100)
#' getMI(x_1, x_2)
#'
#' @export
getMI <- function(x_1, x_2, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(x_1) || length(x_1) == 0) {
    stop("Input x_1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_2) || length(x_2) == 0) {
    stop("Input x_2 must be a non-empty numeric vector.")
  }

  # Check that x_1 and x_2 are of the same length
  if (length(x_1) != length(x_2)) {
    stop("Vectors x_1 and x_2 must be of the same length.")
  }

  # Get the length of the vectors
  n <- length(x_1)

  # Check parameters and generate automatically if incorrect
  params <- resolve_params(n, bin, sp_order)

  bin=as.integer(params$bin)
  sp_order=as.integer(params$sp_order)

  # Call the C functions to compute the mutual information
  out <- .C("compute_mutual_information",
            x_1 = as.double(x_1),
            x_2 = as.double(x_2),
            n = as.integer(n),
            bin = bin,
            sp_order = sp_order,
            eOut = as.double(0))

  # Extract the mutual information value
  mi <- out$eOut

  return(mi)
}


#' @title Calculate Joint Mutual Information
#' @description Computes the joint mutual information I(x_1, x_2; x_3)
#' using the specified number of bins and spline order.
#'
#' @param x_1 A numeric vector for the first variable.
#' @param x_2 A numeric vector for the second variable. Must match the length of `x_1`.
#' @param x_3 A numeric vector for the third variable. Must match the length of `x_1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric value representing joint mutual information (MI).
#'
#' @examples
#' x_1 <- rnorm(100)
#' x_2 <- rnorm(100)
#' x_3 <- rnorm(100)
#' getMIBi(x_1, x_2, x_3)
#'
#' @export
getMIBi <- function(x_1, x_2, x_3, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(x_1) || length(x_1) == 0) {
    stop("Input x_1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_2) || length(x_2) == 0) {
    stop("Input x_2 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_3) || length(x_3) == 0) {
    stop("Input x_3 must be a non-empty numeric vector.")
  }

  # Check that x_1, x_2 and x_3 are of the same length
  if (length(x_1) != length(x_2)) {
    stop("Vectors x_1, x_2, and x_3 must be of the same length.")
  }

  if (length(x_2) != length(x_3)) {
    stop("Vectors x_1, x_2, and x_3 must be of the same length.")
  }

  # Get the length of the vectors
  n <- length(x_1)

  # Check parameters and generate automatically if incorrect
  params <- resolve_params(n, bin, sp_order)

  bin=as.integer(params$bin)
  sp_order=as.integer(params$sp_order)

  # Call the C functions to compute the mutual information
  out <- .C("compute_mutual_information_12_to_3",
            x_1 = as.double(x_1),
            x_2 = as.double(x_2),
            x_3 = as.double(x_3),
            n = as.integer(n),
            bin = bin,
            sp_order = sp_order,
            eOut = as.double(0))

  # Extract the mutual information value
  mi <- out$eOut

  return(mi)
}


#' @title Calculate Conditional Mutual Information
#' @description Computes the conditional mutual information I(x_1; x_2 | x_3)
#' using the specified number of bins and spline order.
#'
#' @param x_1 A numeric vector for the first variable.
#' @param x_2 A numeric vector for the second variable. Must match `x_1` length.
#' @param x_3 A numeric vector for the condition variable. Must match `x_1` length.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric value representing the conditional mutual information (CMI).
#'
#' @examples
#' x_1 <- rnorm(100)
#' x_2 <- rnorm(100)
#' x_3 <- rnorm(100)
#' getCMI(x_1, x_2, x_3)
#'
#' @export
getCMI <- function(x_1, x_2, x_3, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(x_1) || length(x_1) == 0) {
    stop("Input x_1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_2) || length(x_2) == 0) {
    stop("Input x_2 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_3) || length(x_3) == 0) {
    stop("Input x_3 must be a non-empty numeric vector.")
  }

  # Check that x_1, x_2 and x_3 are of the same length
  if (length(x_1) != length(x_2)) {
    stop("Vectors x_1, x_2, and x_3 must be of the same length.")
  }

  if (length(x_2) != length(x_3)) {
    stop("Vectors x_1, x_2, and x_3 must be of the same length.")
  }

  # Get the length of the vectors
  n <- length(x_1)

  # Check parameters and generate automatically if incorrect
  params <- resolve_params(n, bin, sp_order)

  bin=as.integer(params$bin)
  sp_order=as.integer(params$sp_order)

  # Call the C functions to compute the conditional mutual information
  out <- .C("compute_conditional_mutual_information_12_to_3",
            x_1 = as.double(x_1),
            x_2 = as.double(x_2),
            x_3 = as.double(x_3),
            n = as.integer(n),
            bin = bin,
            sp_order = sp_order,
            eOut = as.double(0))

  # Extract the conditional mutual information value
  mi <- out$eOut

  return(mi)
}


#' @title Calculate Bivariate Conditional Mutual Information
#' @description Computes conditional mutual information I(x_1; x_2 | x_3, x_4)
#' using the specified number of bins and spline order.
#'
#' @param x_1 A numeric vector for the first variable.
#' @param x_2 A numeric vector for the second variable. Must match `x_1` length.
#' @param x_3 A numeric vector for the first condition variable. Must match `x_1` length.
#' @param x_4 A numeric vector for the second condition variable. Must match `x_1` length.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric value representing the bivariate conditional mutual information (CMI).
#'
#' @examples
#' x_1 <- rnorm(100)
#' x_2 <- rnorm(100)
#' x_3 <- rnorm(100)
#' x_4 <- rnorm(100)
#' getCMIBiCondi(x_1, x_2, x_3, x_4)
#'
#' @export
getCMIBiCondi <- function(x_1, x_2, x_3, x_4, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(x_1) || length(x_1) == 0) {
    stop("Input x_1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_2) || length(x_2) == 0) {
    stop("Input x_2 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_3) || length(x_3) == 0) {
    stop("Input x_3 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_4) || length(x_4) == 0) {
    stop("Input x_4 must be a non-empty numeric vector.")
  }

  # Check that x_1, x_2, x_3 and x_4 are of the same length
  if (length(x_1) != length(x_2)) {
    stop("Vectors x_1, x_2, x_3 and x_4 must be of the same length.")
  }

  if (length(x_2) != length(x_3)) {
    stop("Vectors x_1, x_2, x_3 and x_4 must be of the same length.")
  }

  if (length(x_3) != length(x_4)) {
    stop("Vectors x_1, x_2, x_3 and x_4 must be of the same length.")
  }

  # Get the length of the vectors
  n <- length(x_1)

  # Check parameters and generate automatically if incorrect
  params <- resolve_params(n, bin, sp_order)

  bin=as.integer(params$bin)
  sp_order=as.integer(params$sp_order)

  # Call the C functions to compute the conditional mutual information
  out <- .C("compute_conditional_mutual_information_12_to_34",
            x_1 = as.double(x_1),
            x_2 = as.double(x_2),
            x_3 = as.double(x_3),
            x_3 = as.double(x_4),
            n = as.integer(n),
            bin = bin,
            sp_order = sp_order,
            eOut = as.double(0))

  # Extract the conditional mutual information value
  mi <- out$eOut

  return(mi)
}

#' @title Calculate P-Value from a Sorted Distribution
#' @description Computes the P-value for a given numeric value `x` based on its position within a sorted distribution.
#' This function utilizes a binary search approach for efficient computation.
#'
#' @param x A numeric value for which the P-value is to be calculated.
#' @param sorted_Distri A numeric vector representing a sorted distribution. This distribution must be sorted in ascending order.
#'
#' @return A numeric value indicating the P-value, representing the proportion of values in `sorted_Distri`
#' that are greater than or equal to `x`.
#'
#' @examples
#' x <- rnorm(1)
#' sorted_dist <- sort(rnorm(100))
#' getPValue(x, sorted_dist)
#'
#' @export
getPValue <- function(x,sorted_Distri) {
  sizeD = length(sorted_Distri)
  if (x>sorted_Distri[sizeD]) {return(0)}
  if (x<=sorted_Distri[1]) {return(sizeD)}
  beginIdx = 1;endIdx = sizeD;
  while ((endIdx-beginIdx)!=1) {
    midIdx = ceiling((endIdx+beginIdx)/2)
    if(sorted_Distri[midIdx]<x) {beginIdx = midIdx}
    else {endIdx = midIdx}
  }
  if (sorted_Distri[endIdx]==x) {result = (sizeD-endIdx+1)}
  else (result = (sizeD-beginIdx))

  return(result/sizeD)
}

#' @title Internal Utility Function for Random Sampling Loops
#' @description Executes nested loops to perform random sampling operations.
#' This function is intended for internal use only and should not be accessed by package users.
#' @keywords internal
#'
#' @return No return value. This function is designed to allow large permutation tests distributed to multiple nodes.
additionalRunIF <- function(x, y, z, p){
  for (i in 1:x) {
    for (j in 1:y) {
      for (k in 1:z) {
        runif(p)
      }
    }
  }
}

#' @title Permuted Correlation Between Matrix and Vector
#' @description Computes the correlation between a randomly sampled vector from a matrix and a given vector.
#' The sampling is done multiple times to generate a distribution.
#'
#' @param mat A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param vec A numeric vector, with length equal to the number of columns in `mat`.
#' @param cor_type Type of correlation to calculate: "Pearson", "Kendall", or "Spearman". Default is "Pearson".
#' @param bulkIdx Index to divide the task when processing many permutations. Default is 0.
#' @param permutationTimes Number of permutations for sampling. Default is 1000.
#' @param seedNum Seed for random number generation. Default is 99999999.
#' @return A numeric vector of correlation values for each permutation.
#'
#' @examples
#' mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' vec <- rnorm(100)
#' CORmat2vecPermu(mat, vec)
#'
#' @export
CORmat2vecPermu <- function(mat, vec, cor_type = "pearson", bulkIdx = 0, permutationTimes = 1000, seedNum = 99999999){
  set.seed(seedNum)

  if (!is.numeric(vec) || length(vec) == 0) {
    stop("Input vec must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat) || !is.numeric(mat) || is.na(sum(mat)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that mat and vec are of the same length
  if (ncol(mat) != length(vec)) {
    stop("The column of mat and the length of vec must be the same.")
  }

  if (bulkIdx!=0) {
    additionalRunIF(bulkIdx,ncol(mat),2,permutationTimes)
  }

  permutationCorVector <- rep(0,permutationTimes)
  for (p in 1:permutationTimes) {
    randRowIdx <- sample(1:nrow(mat),size = ncol(mat),replace = (ncol(mat)>nrow(mat)) )
    randColIdx <- sample(1:ncol(mat),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp1 <- mat[idx]
    permutationCorVector[p] <- cor(randomExp1, vec, method = cor_type)
  }

  return(permutationCorVector)
}

#' @title Permuted Normalized Mutual Information Between Matrix and Vector
#' @description Computes the mutual information (MI) between a random vector sampled from a matrix and a vector,
#' normalized by the mutual information of the vector with itself. The sampling is done multiple times to generate a distribution.
#'
#' @param mat A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param vec A numeric vector, with length equal to the number of columns in `mat`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @param bulkIdx Index to divide the task when processing many permutations. Default is 0.
#' @param permutationTimes Number of permutations for sampling. Default is 1000.
#' @param seedNum Seed for random number generation. Default is 99999999.
#' @return A numeric vector of normalized mutual information (MI) values for each permutation.
#'
#' @examples
#' mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' vec <- rnorm(100)
#' MImat2vecPermu(mat, vec)
#'
#' @export
MImat2vecPermu <- function(mat, vec, bin = NULL, sp_order = NULL, bulkIdx = 0, permutationTimes = 1000, seedNum = 99999999){
  set.seed(seedNum)

  if (!is.numeric(vec) || length(vec) == 0) {
    stop("Input vec must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat) || !is.numeric(mat) || is.na(sum(mat)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that mat and vec are of the same length
  if (ncol(mat) != length(vec)) {
    stop("The column of mat and the length of vector vec must be the same.")
  }

  max_MI = getMI(vec, vec, bin = bin, sp_order = sp_order)
  # Ensure vec contains information
  if (max_MI == 0) {
    stop("Vector 'vec' must contain information.")
  }

  if (bulkIdx!=0) {
    additionalRunIF(bulkIdx,ncol(mat),2,permutationTimes)
  }

  permutationMIVector <- rep(0,permutationTimes)
  for (p in 1:permutationTimes) {
    randRowIdx <- sample(1:nrow(mat),size = ncol(mat),replace = (ncol(mat)>nrow(mat)) )
    randColIdx <- sample(1:ncol(mat),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp1 <- mat[idx]

    permutationMIVector[p] <- getMI(randomExp1, vec, bin = bin, sp_order = sp_order)
  }
  permutationMIVector <- permutationMIVector/max_MI
  permutationMIVector[permutationMIVector<0] = 0
  return(permutationMIVector)
}

#' @title Permuted Normalized Conditional Mutual Information Between Matrix and Vector
#' @description Computes the conditional mutual information (CMI) between a random vector sampled from a matrix and a vector,
#' conditioned on a third vector, normalized by the mutual information of the vector with itself.
#' The sampling is done multiple times to generate a distribution.
#'
#' @param mat A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param vec A numeric vector, with length equal to the number of columns in `mat`.
#' @param condi A numeric condition vector, matching the number of columns in `mat`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @param bulkIdx Index to divide the task when processing many permutations. Default is 0.
#' @param permutationTimes Number of permutations for sampling. Default is 1000.
#' @param seedNum Seed for random number generation. Default is 99999999.
#' @return A numeric vector of normalized conditional mutual information (CMI) values for each permutation.
#'
#' @examples
#' mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' vec <- rnorm(100)
#' condi <- rnorm(100)
#' CMImat2vecPermu(mat, vec, condi)
#'
#' @export
CMImat2vecPermu <- function(mat, vec, condi, bin = NULL, sp_order = NULL, bulkIdx = 0, permutationTimes = 1000, seedNum = 99999999){
  set.seed(seedNum)

  if (!is.numeric(vec) || length(vec) == 0) {
    stop("Input vec must be a non-empty numeric vector.")
  }

  if (!is.numeric(condi) || length(condi) == 0) {
    stop("Input condi must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat) || !is.numeric(mat) || is.na(sum(mat)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that mat and vec are of the same length
  if (ncol(mat) != length(vec)) {
    stop("The column of mat and the length of vec must be the same.")
  }

  if (ncol(mat) != length(condi)) {
    stop("The column of mat and the length of vector condi must be the same.")
  }

  max_MI = getMI(vec, vec, bin = bin, sp_order = sp_order)
  if (max_MI == 0) {
    stop("Vector 'vec' must contain information.")
  }

  if (bulkIdx!=0) {
    additionalRunIF(bulkIdx,ncol(mat),2,permutationTimes)
  }

  permutationCondiMIVector <- rep(0,permutationTimes)
  for (p in 1:permutationTimes) {
    randRowIdx <- sample(1:nrow(mat),size = ncol(mat),replace = (ncol(mat)>nrow(mat)) )
    randColIdx <- sample(1:ncol(mat),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp1 <- mat[idx]

    permutationCondiMIVector[p] <- getCMI(randomExp1, vec, condi, bin = bin, sp_order = sp_order)
  }
  permutationCondiMIVector <- permutationCondiMIVector/max_MI
  permutationCondiMIVector[permutationCondiMIVector<0] = 0
  return(permutationCondiMIVector)
}

#' @title Permuted Mutual Information Between Two Matrices
#' @description Computes the normalized mutual information (MI) between vectors sampled from two matrices normalized by the individual information content.
#' The sampling is done multiple times to generate a distribution.
#'
#' @param mat1 A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param mat2 Another numeric matrix to compare against. Must have the same dimensions as `mat1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @param bulkIdx Index to divide the task when processing many permutations. Default is 0.
#' @param permutationTimes Number of permutations for sampling. Default is 1000.
#' @param seedNum Seed for random number generation. Default is 99999999.
#' @return A numeric vector of normalized mutual information (MI) values for each permutation.
#'
#' @examples
#' mat1 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' mat2 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' MImat2matPermu(mat1, mat2)
#'
#' @export
MImat2matPermu <- function(mat1, mat2, bin = NULL, sp_order = NULL, bulkIdx = 0, permutationTimes = 1000, seedNum = 99999999){
  set.seed(seedNum)

  # Validate inputs as non-empty numeric vectors

  if (!is.matrix(mat1) || !is.numeric(mat1) || is.na(sum(mat1)) ) {
    stop("Input mat1 must be a non-NA numeric matrix.")
  }

  if (!is.matrix(mat2) || !is.numeric(mat2) || is.na(sum(mat2)) ) {
    stop("Input mat2 must be a non-NA numeric matrix.")
  }

  # Check that 2 mats are of the same size
  if (ncol(mat1) != ncol(mat2) ) {
    stop("The column size of mat1 and mat2 must be the same.")
  }

  if (bulkIdx!=0) {
    additionalRunIF(bulkIdx,ncol(mat1),4,permutationTimes)
  }

  permutationMIVector <- rep(0,permutationTimes)

  for (p in 1:permutationTimes) {
    randRowIdx <- sample(1:nrow(mat1),size = ncol(mat1),replace = (ncol(mat1)>nrow(mat1)) )
    randColIdx <- sample(1:ncol(mat1),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp1 <- mat1[idx]
    randRowIdx <- sample(1:nrow(mat2),size = ncol(mat2),replace = (ncol(mat2)>nrow(mat2)) )
    randColIdx <- sample(1:ncol(mat2),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp2 <- mat2[idx]
    permutationMIVector[p] <- getMI(randomExp1,randomExp2, bin = bin, sp_order = sp_order) /
      max(getMI(randomExp1,randomExp1, bin = bin, sp_order = sp_order),
          getMI(randomExp2,randomExp2, bin = bin, sp_order = sp_order))

  }
  permutationMIVector[permutationMIVector<0] = 0
  return(permutationMIVector)
}

#' @title Permuted Conditional Mutual Information Between Two Matrices
#' @description Computes the normalized conditional mutual information (CMI) between vectors sampled from two matrices,
#' conditioned on another vector, normalized by the individual information content. The sampling is done multiple times to generate a distribution.
#'
#' @param mat1 A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param mat2 Another numeric matrix to compare against. Must have the same dimensions as `mat1`.
#' @param condi A numeric condition vector, matching the number of columns in `mat1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @param bulkIdx Index to divide the task when processing many permutations. Default is 0.
#' @param permutationTimes Number of permutations for sampling. Default is 1000.
#' @param seedNum Seed for random number generation. Default is 99999999.
#' @return A numeric vector of normalized conditional mutual information (CMI) values for each permutation.
#'
#' @examples
#' mat1 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' mat2 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' condi <- rnorm(100)
#' CMImat2matPermu(mat1, mat2, condi)
#'
#' @export
CMImat2matPermu <- function(mat1, mat2, condi, bin = NULL, sp_order = NULL, bulkIdx = 0, permutationTimes = 1000, seedNum = 99999999){
  set.seed(seedNum)

  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(condi) || length(condi) == 0) {
    stop("Input condi must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat1) || !is.numeric(mat1) || is.na(sum(mat1)) ) {
    stop("Input mat1 must be a non-NA numeric matrix.")
  }

  if (!is.matrix(mat2) || !is.numeric(mat2) || is.na(sum(mat2)) ) {
    stop("Input mat2 must be a non-NA numeric matrix.")
  }

  # Check that 2 mats are of the same size
  if (ncol(mat1) != ncol(mat2) ) {
    stop("The column size of mat1 and mat2 must be the same.")
  }

  if (ncol(mat1) != length(condi)) {
    stop("The column of mat1 and the length of vector condi must be the same.")
  }

  if (bulkIdx!=0) {
    additionalRunIF(bulkIdx,ncol(mat1),4,permutationTimes)
  }

  permutationMIVector <- rep(0,permutationTimes)

  for (p in 1:permutationTimes) {
    randRowIdx <- sample(1:nrow(mat1),size = ncol(mat1),replace = (ncol(mat1)>nrow(mat1)) )
    randColIdx <- sample(1:ncol(mat1),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp1 <- mat1[idx]
    randRowIdx <- sample(1:nrow(mat2),size = ncol(mat2),replace = (ncol(mat2)>nrow(mat2)) )
    randColIdx <- sample(1:ncol(mat2),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp2 <- mat2[idx]
    permutationMIVector[p] <- getCMI(randomExp1,randomExp2, condi, bin = bin, sp_order = sp_order) /
      max(getMI(randomExp1,randomExp1, bin = bin, sp_order = sp_order),
          getMI(randomExp2,randomExp2, bin = bin, sp_order = sp_order))

  }
  permutationMIVector[permutationMIVector<0] = 0
  return(permutationMIVector)
}


#' @title Permuted Normalized Conditional Mutual Information Between Matrix and Vector Given Two Conditions
#' @description Computes the conditional mutual information (CMI) between a random vector sampled from a matrix and a vector,
#' conditioned on two vectors, normalized by the mutual information of the vector with itself.
#' The sampling is done multiple times to generate a distribution.
#'
#' @param mat A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param vec A numeric vector, with length equal to the number of columns in `mat`.
#' @param condi1 A numeric condition vector, matching the number of columns in `mat`.
#' @param condi2 Another numeric condition vector, matching the number of columns in `mat`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @param bulkIdx Index to divide the task when processing many permutations. Default is 0.
#' @param permutationTimes Number of permutations for sampling. Default is 1000.
#' @param seedNum Seed for random number generation. Default is 99999999.
#' @return A numeric vector of normalized conditional mutual information (CMI) values for each permutation.
#'
#' @examples
#' mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' vec <- rnorm(100)
#' condi1 <- rnorm(100)
#' condi2 <- rnorm(100)
#' CMIBiCondimat2vecPermu(mat, vec, condi1, condi2)
#'
#' @export
CMIBiCondimat2vecPermu <- function(mat, vec, condi1, condi2, bin = NULL, sp_order = NULL, bulkIdx = 0, permutationTimes = 1000, seedNum = 99999999){
  set.seed(seedNum)

  if (!is.numeric(vec) || length(vec) == 0) {
    stop("Input vec must be a non-empty numeric vector.")
  }

  if (!is.numeric(condi1) || length(condi1) == 0) {
    stop("Input condi1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(condi2) || length(condi2) == 0) {
    stop("Input condi2 must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat) || !is.numeric(mat) || is.na(sum(mat)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that mat and vec are of the same length
  if (ncol(mat) != length(vec)) {
    stop("The column of mat and the length of vec must be the same.")
  }

  if (ncol(mat) != length(condi1)) {
    stop("The column of mat and the length of vector condi1 must be the same.")
  }

  if (ncol(mat) != length(condi2)) {
    stop("The column of mat and the length of vector condi2 must be the same.")
  }

  max_MI = getMI(vec, vec, bin = bin, sp_order = sp_order)
  if (max_MI == 0) {
    stop("Vector 'vec' must contain information.")
  }

  if (bulkIdx!=0) {
    additionalRunIF(bulkIdx,ncol(mat),2,permutationTimes)
  }

  permutationCondiMIVector <- rep(0,permutationTimes)
  for (p in 1:permutationTimes) {
    randRowIdx <- sample(1:nrow(mat),size = ncol(mat),replace = (ncol(mat)>nrow(mat)) )
    randColIdx <- sample(1:ncol(mat),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp1 <- mat[idx]

    permutationCondiMIVector[p] <- getCMIBiCondi(randomExp1, vec, condi1, condi2, bin = bin, sp_order = sp_order)
  }
  permutationCondiMIVector <- permutationCondiMIVector/max_MI
  permutationCondiMIVector[permutationCondiMIVector<0] = 0
  return(permutationCondiMIVector)
}


#' @title Permuted Conditional Mutual Information Between Two Matrices Given Two Conditions
#' @description Computes the normalized conditional mutual information (CMI) between vectors sampled from two matrices,
#' conditioned on two vectors, normalized by the individual information content.
#' The sampling is done multiple times to generate a distribution.
#'
#' @param mat1 A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param mat2 Another numeric matrix to compare against. Must have the same dimensions as `mat1`.
#' @param condi1 A numeric condition vector, matching the number of columns in `mat1`.
#' @param condi2 Another numeric condition vector, matching the number of columns in `mat`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @param bulkIdx Index to divide the task when processing many permutations. Default is 0.
#' @param permutationTimes Number of permutations for sampling. Default is 1000.
#' @param seedNum Seed for random number generation. Default is 99999999.
#' @return A numeric vector of normalized conditional mutual information (CMI) values for each permutation.
#'
#' @examples
#' mat1 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' mat2 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' condi1 <- rnorm(100)
#' condi2 <- rnorm(100)
#' CMIBiCondimat2matPermu(mat1, mat2, condi1, condi2)
#'
#' @export
CMIBiCondimat2matPermu <- function(mat1, mat2, condi1, condi2, bin = NULL, sp_order = NULL, bulkIdx = 0, permutationTimes = 1000, seedNum = 99999999){
  set.seed(seedNum)

  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(condi1) || length(condi1) == 0) {
    stop("Input condi1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(condi2) || length(condi2) == 0) {
    stop("Input condi2 must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat1) || !is.numeric(mat1) || is.na(sum(mat1)) ) {
    stop("Input mat1 must be a non-NA numeric matrix.")
  }

  if (!is.matrix(mat2) || !is.numeric(mat2) || is.na(sum(mat2)) ) {
    stop("Input mat2 must be a non-NA numeric matrix.")
  }

  # Check that 2 mats are of the same size
  if (ncol(mat1) != ncol(mat2) ) {
    stop("The column size of mat1 and mat2 must be the same.")
  }

  if (ncol(mat1) != length(condi1)) {
    stop("The column of mat1 and the length of vector condi1 must be the same.")
  }

  if (ncol(mat1) != length(condi2)) {
    stop("The column of mat1 and the length of vector condi2 must be the same.")
  }

  if (bulkIdx!=0) {
    additionalRunIF(bulkIdx,ncol(mat1),4,permutationTimes)
  }

  permutationMIVector <- rep(0,permutationTimes)

  for (p in 1:permutationTimes) {
    randRowIdx <- sample(1:nrow(mat1),size = ncol(mat1),replace = (ncol(mat1)>nrow(mat1)) )
    randColIdx <- sample(1:ncol(mat1),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp1 <- mat1[idx]
    randRowIdx <- sample(1:nrow(mat2),size = ncol(mat2),replace = (ncol(mat2)>nrow(mat2)) )
    randColIdx <- sample(1:ncol(mat2),replace = F)
    idx <- cbind(randRowIdx, randColIdx)
    randomExp2 <- mat2[idx]
    permutationMIVector[p] <- getCMIBiCondi(randomExp1,randomExp2, condi1, condi2, bin = bin, sp_order = sp_order) /
      max(getMI(randomExp1,randomExp1, bin = bin, sp_order = sp_order),
          getMI(randomExp2,randomExp2, bin = bin, sp_order = sp_order))

  }
  permutationMIVector[permutationMIVector<0] = 0
  return(permutationMIVector)
}

#' @title Normalized Mutual Information Between Matrix and Vector
#'
#' @description Computes the normalized mutual information (MI) between each row of a matrix and
#' a numeric vector normalized by the mutual information of the vector with itself using the specified number of bins and spline order.
#'
#' @param mat A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param vec A numeric vector, with length equal to the number of columns in `mat`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#'
#' @return A numeric vector representing the normalized mutual information (MI) between each row of `mat` and `vec`.
#'
#' @examples
#' mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' vec <- rnorm(100)
#' MImat2vec(mat, vec)
#'
#' @export
MImat2vec <- function(mat, vec, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(vec) || length(vec) == 0) {
    stop("Input vec must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat) || !is.numeric(mat) || is.na(sum(mat)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that mat and vec are of the same length
  if (ncol(mat) != length(vec)) {
    stop("The column of mat and the length of vector vec must be the same.")
  }

  max_MI <- getMI(vec, vec, bin = bin, sp_order = sp_order)
  # Ensure vec contains information
  if (max_MI == 0) {
    stop("Vector 'vec' must contain information.")
  }

  mi_vector <- sapply(1:nrow(mat), function(x)
    getMI(mat[x,], vec, bin = bin, sp_order = sp_order))

  normalized_mi_vector <- mi_vector / max_MI
  names(normalized_mi_vector) <- row.names(mat)

  return(normalized_mi_vector)
}


#' @title Normalized Conditional Mutual Information Between Matrix and Vector
#' @description Computes the normalized conditional mutual information (CMI) between each row of a matrix and a vector,
#' given a third condition vector, normalized by the mutual information of the vector with itself using the specified bins and spline order.
#'
#' @param mat A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param vec A numeric vector, with length equal to the number of columns in `mat`.
#' @param condi A numeric condition vector, matching the number of columns in `mat`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric vector representing the normalized conditional mutual information (CMI) between each row of `mat` and `vec`, given `condi`.
#'
#' @examples
#' mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' vec <- rnorm(100)
#' condi <- rnorm(100)
#' CMImat2vec(mat, vec, condi)
#'
#' @export
CMImat2vec <- function(mat, vec, condi, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(vec) || length(vec) == 0) {
    stop("Input vec must be a non-empty numeric vector.")
  }

  if (!is.numeric(condi) || length(condi) == 0) {
    stop("Input condi must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat) || !is.numeric(mat) || is.na(sum(mat)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that mat, vec and condi are of the same length
  if (ncol(mat) != length(vec)) {
    stop("The column of mat and the length of vec must be the same.")
  }

  if (ncol(mat) != length(condi)) {
    stop("The column of mat and the length of vector condi must be the same.")
  }

  max_MI <- getMI(vec, vec, bin = bin, sp_order = sp_order)
  # Ensure vec contains information
  if (max_MI == 0) {
    stop("Vector 'vec' must contain information.")
  }

  condi_mi_vector <- sapply(1:nrow(mat), function(x)
    getCMI(mat[x,], vec, condi, bin = bin, sp_order = sp_order))

  normalized_condi_mi_vector <- condi_mi_vector / max_MI
  names(normalized_condi_mi_vector) <- row.names(mat)

  return(normalized_condi_mi_vector)
}


#' @title Normalized Mutual Information Between Two Matrices
#' @description Computes the normalized mutual information (MI) between the same rows of two matrices
#' normalized by their individual information content, using the specified number of bins and spline order.
#'
#' @param mat1 A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param mat2 Another numeric matrix to compare against. Must have the same dimensions as `mat1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric vector where each element corresponds to the normalized mutual information (MI) between
#' respective rows of `mat1` and `mat2`.
#'
#' @examples
#' mat1 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' mat2 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' MImat2mat(mat1, mat2)
#'
#' @export
MImat2mat <- function(mat1, mat2, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.matrix(mat1) || !is.numeric(mat1) || is.na(sum(mat1)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  if (!is.matrix(mat2) || !is.numeric(mat2) || is.na(sum(mat2)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that 2 mats are of the same size
  if (ncol(mat1) != ncol(mat2) | nrow(mat1) != nrow(mat2) ) {
    stop("The size of mat1 and mat2 must be the same.")
  }

  max_MI <- sapply(1:nrow(mat1), function(x)
    max(getMI(mat1[x,], mat1[x,], bin = bin, sp_order = sp_order),
        getMI(mat2[x,], mat2[x,], bin = bin, sp_order = sp_order))
  )

  mi_vector <- sapply(1:nrow(mat1), function(x)
    getMI(mat1[x,], mat2[x,], bin = bin, sp_order = sp_order))

  normalized_mi_vector <- ifelse(max_MI == 0, 0, mi_vector / max_MI)
  names(normalized_mi_vector) <- row.names(mat1)
  normalized_mi_vector[normalized_mi_vector<0] = 0
  return(normalized_mi_vector)
}


#' @title Normalized Conditional Mutual Information Between Two Matrices
#' @description Computes the normalized conditional mutual information (CMI) between the same rows of two matrices,
#' given a condition variable, normalized by their individual information content. CMI is calculated using the specified number of bins and spline order.
#'
#' @param mat1 A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param mat2 Another numeric matrix to compare against. Must have the same dimensions as `mat1`.
#' @param condi A numeric condition vector, matching the number of columns in `mat1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric vector representing the normalized conditional mutual information (CMI) between pairs of rows
#' from `mat1` and `mat2`, conditioned on `condi`.
#'
#' @examples
#' mat1 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' mat2 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' condi <- rnorm(100)
#' CMImat2mat(mat1, mat2, condi)
#'
#' @export
CMImat2mat <- function(mat1, mat2, condi, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(condi) || length(condi) == 0) {
    stop("Input condi must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat1) || !is.numeric(mat1) || is.na(sum(mat1)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  if (!is.matrix(mat2) || !is.numeric(mat2) || is.na(sum(mat2)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that 2 mats are of the same size
  if (ncol(mat1) != ncol(mat2) | nrow(mat1) != nrow(mat2) ) {
    stop("The size of mat1 and mat2 must be the same.")
  }

  if (ncol(mat1) != length(condi)) {
    stop("The column of mat1 and the length of vector condi must be the same.")
  }

  max_MI <- sapply(1:nrow(mat1), function(x)
    max(getMI(mat1[x,], mat1[x,], bin = bin, sp_order = sp_order),
        getMI(mat2[x,], mat2[x,], bin = bin, sp_order = sp_order))
  )

  mi_vector <- sapply(1:nrow(mat1), function(x)
    getCMI(mat1[x,], mat2[x,], condi, bin = bin, sp_order = sp_order))

  normalized_mi_vector <- ifelse(max_MI == 0, 0, mi_vector / max_MI)
  names(normalized_mi_vector) <- row.names(mat1)
  normalized_mi_vector[normalized_mi_vector<0] = 0
  return(normalized_mi_vector)
}

#' @title Normalized Conditional Mutual Information Between Matrix and Vector Given Two Conditions
#' @description Computes the normalized conditional mutual information (CMI) between each row of a matrix and a vector,
#' given two condition vectors, normalized by the mutual information of the vector with itself using the specified bins and spline order.
#'
#' @param mat A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param vec A numeric vector, with length equal to the number of columns in `mat`.
#' @param condi1 A numeric condition vector, matching the number of columns in `mat`.
#' @param condi2 Another numeric condition vector, matching the number of columns in `mat`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric vector representing the normalized conditional mutual information (CMI) between each row of `mat` and `vec`, given `condi1` and `condi2`.
#'
#' @examples
#' mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' vec <- rnorm(100)
#' condi1 <- rnorm(100)
#' condi2 <- rnorm(100)
#' CMIBiCondimat2vec(mat, vec, condi1, condi2)
#'
#' @export
CMIBiCondimat2vec <- function(mat, vec, condi1, condi2, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(vec) || length(vec) == 0) {
    stop("Input vec must be a non-empty numeric vector.")
  }

  if (!is.numeric(condi1) || length(condi1) == 0) {
    stop("Input condi1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(condi2) || length(condi2) == 0) {
    stop("Input condi2 must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat) || !is.numeric(mat) || is.na(sum(mat)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that mat, vec and condi1 are of the same length
  if (ncol(mat) != length(vec)) {
    stop("The column of mat and the length of vec must be the same.")
  }

  if (ncol(mat) != length(condi1)) {
    stop("The column of mat and the length of vector condi1 must be the same.")
  }

  if (ncol(mat) != length(condi2)) {
    stop("The column of mat and the length of vector condi2 must be the same.")
  }

  max_MI <- getMI(vec, vec, bin = bin, sp_order = sp_order)
  # Ensure vec contains information
  if (max_MI == 0) {
    stop("Vector 'vec' must contain information.")
  }

  condi_mi_vector <- sapply(1:nrow(mat), function(x)
    getCMIBiCondi(mat[x,], vec, condi1, condi2, bin = bin, sp_order = sp_order))

  normalized_condi_mi_vector <- condi_mi_vector / max_MI
  names(normalized_condi_mi_vector) <- row.names(mat)

  return(normalized_condi_mi_vector)
}


#' @title Normalized Conditional Mutual Information Between Two Matrices Given Two Conditions
#' @description Computes the normalized conditional mutual information (CMI) between the same rows of two matrices,
#' given two condition variables, normalized by their individual information content. CMI is calculated using the specified number of bins and spline order.
#'
#' @param mat1 A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param mat2 Another numeric matrix to compare against. Must have the same dimensions as `mat1`.
#' @param condi1 A numeric condition vector, matching the number of columns in `mat1`.
#' @param condi2 Another numeric condition vector, matching the number of columns in `mat1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric vector representing the normalized conditional mutual information (CMI) between pairs of rows
#' from `mat1` and `mat2`, conditioned on `condi1` and `condi2`.
#'
#' @examples
#' mat1 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' mat2 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
#' condi1 <- rnorm(100)
#' condi2 <- rnorm(100)
#' CMIBiCondimat2mat(mat1, mat2, condi1, condi2)
#'
#' @export
CMIBiCondimat2mat <- function(mat1, mat2, condi1, condi2, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(condi1) || length(condi1) == 0) {
    stop("Input condi1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(condi2) || length(condi2) == 0) {
    stop("Input condi2 must be a non-empty numeric vector.")
  }

  if (!is.matrix(mat1) || !is.numeric(mat1) || is.na(sum(mat1)) ) {
    stop("Input mat1 must be a non-NA numeric matrix.")
  }

  if (!is.matrix(mat2) || !is.numeric(mat2) || is.na(sum(mat2)) ) {
    stop("Input mat2 must be a non-NA numeric matrix.")
  }

  # Check that 2 mats are of the same size
  if (ncol(mat1) != ncol(mat2) | nrow(mat1) != nrow(mat2) ) {
    stop("The size of mat1 and mat2 must be the same.")
  }

  if (ncol(mat1) != length(condi1)) {
    stop("The column of mat1 and the length of vector condi1 must be the same.")
  }

  if (ncol(mat1) != length(condi2)) {
    stop("The column of mat1 and the length of vector condi2 must be the same.")
  }

  max_MI <- sapply(1:nrow(mat1), function(x)
    max(getMI(mat1[x,], mat1[x,], bin = bin, sp_order = sp_order),
        getMI(mat2[x,], mat2[x,], bin = bin, sp_order = sp_order))
  )

  mi_vector <- sapply(1:nrow(mat1), function(x)
    getCMIBiCondi(mat1[x,], mat2[x,], condi1, condi2, bin = bin, sp_order = sp_order))

  normalized_mi_vector <- ifelse(max_MI == 0, 0, mi_vector / max_MI)
  names(normalized_mi_vector) <- row.names(mat1)
  normalized_mi_vector[normalized_mi_vector<0] = 0
  return(normalized_mi_vector)
}


#' @title Normalized Mutual Information Between Each Row of Two Matrices
#' @description Computes the normalized mutual information (MI) between each row of first matrix and each row of second matrix,
#' normalized by their individual information content, using the specified number of bins and spline order.
#'
#' @param mat1 A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param mat2 Another numeric matrix to compare against. Must have the columns as `mat1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A matrix where each element corresponds to the normalized mutual information (MI) between
#' respective rows of `mat1` and `mat2`.
#'
#' @examples
#' mat1 <- matrix(rnorm(1000), nrow = 10, ncol = 100)
#' mat2 <- matrix(rnorm(2000), nrow = 200, ncol = 100)
#' MImat2matAll(mat1, mat2)
#'
#' @export
MImat2matAll <- function(mat1, mat2, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.matrix(mat1) || !is.numeric(mat1) || is.na(sum(mat1)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  if (!is.matrix(mat2) || !is.numeric(mat2) || is.na(sum(mat2)) ) {
    stop("Input mat must be a non-NA numeric matrix.")
  }

  # Check that 2 mats are of the same size
  if (ncol(mat1) != ncol(mat2) ) {
    stop("The size of mat1 and mat2 must be the same.")
  }

  mi_matrix <- do.call(
    rbind,
    lapply(seq_len(nrow(mat1)), function(i) {
      repeated_row <- matrix(
        rep(mat1[i, ], each = nrow(mat2)),
        nrow = nrow(mat2), byrow = FALSE
      )

      MImat2mat(
        mat2, repeated_row,
        bin = bin,
        sp_order = sp_order
      )
    })
  )

  rownames(mi_matrix) <- rownames(mat1)
  colnames(mi_matrix) <- rownames(mat2)

  return(mi_matrix)
}


#' @title Normalized Conditional Mutual Information Between Each Row of Two Matrices
#' @description Computes the normalized conditional mutual information (CMI) between each row of first matrix and each row of second matrix,
#' given a condition variable, normalized by their individual information content. CMI is calculated using the
#' specified number of bins and spline order.
#'
#' @param mat1 A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param mat2 Another numeric matrix to compare against. Must have the same columns as `mat1`.
#' @param condi A numeric condition vector, matching the number of columns in `mat1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A matrix where each element corresponds to the normalized conditional mutual information (CMI)
#' between each row of `mat1` and each row of `mat2`, conditioned on `condi`.
#'
#' @examples
#' mat1 <- matrix(rnorm(1000), nrow = 10, ncol = 100)
#' mat2 <- matrix(rnorm(20000), nrow = 200, ncol = 100)
#' condi <- rnorm(100)
#' CMImat2matAll(mat1, mat2, condi)
#'
#' @export
CMImat2matAll <- function(mat1, mat2, condi, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(condi) || length(condi) == 0 || is.na(sum(condi)) ) {
    stop("Input condi must be a non-empty non-NA numeric vector.")
  }

  if (!is.matrix(mat1) || !is.numeric(mat1) || is.na(sum(mat1)) ) {
    stop("Input mat1 must be a non-NA numeric matrix.")
  }

  if (!is.matrix(mat2) || !is.numeric(mat2) || is.na(sum(mat2)) ) {
    stop("Input mat2 must be a non-NA numeric matrix.")
  }

  # Check that 2 mats have the same number of columns
  if (ncol(mat1) != ncol(mat2) ) {
    stop("mat1 and mat2 must have the same number of columns.")
  }

  if (ncol(mat1) != length(condi)) {
    stop("The column of mat1 and the length of vector condi must be the same.")
  }

  cmi_matrix <- do.call(
    rbind,
    lapply(seq_len(nrow(mat1)), function(i) {
      repeated_row <- matrix(
        rep(mat1[i, ], each = nrow(mat2)),
        nrow = nrow(mat2),
        byrow = FALSE
      )

      CMImat2mat(
        repeated_row, mat2,
        condi = condi, bin = bin, sp_order = sp_order
      )
    })
  )

  rownames(cmi_matrix) <- rownames(mat1)
  colnames(cmi_matrix) <- rownames(mat2)

  return(cmi_matrix)
}


#' @title Normalized Conditional Mutual Information Between Each Row of Two Matrices Given Two Conditions
#' @description Computes the normalized conditional mutual information (CMI) between each row of first matrix and each row of second matrix,
#' given two condition variables, normalized by their individual information content. CMI is calculated using
#' the specified number of bins and spline order.
#'
#' @param mat1 A numeric matrix. For example, each row represents a gene and each column represents a sample.
#' @param mat2 Another numeric matrix to compare against. Must have the same columns as `mat1`.
#' @param condi1 A numeric condition vector, matching the number of columns in `mat1`.
#' @param condi2 Another numeric condition vector, matching the number of columns in `mat1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A matrix where each element corresponds to the normalized conditional mutual information (CMI)
#' between each row of `mat1` and each row of `mat2`, conditioned on `condi1` and `condi2`.
#'
#' @examples
#' mat1 <- matrix(rnorm(1000), nrow = 10, ncol = 100)
#' mat2 <- matrix(rnorm(20000), nrow = 200, ncol = 100)
#' condi1 <- rnorm(100)
#' condi2 <- rnorm(100)
#' CMIBiCondimat2matAll(mat1, mat2, condi1, condi2)
#'
#' @export
CMIBiCondimat2matAll <- function(mat1, mat2, condi1, condi2, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(condi1) || length(condi1) == 0 || is.na(sum(condi1)) ) {
    stop("Input condi1 must be a non-empty non-NA numeric vector.")
  }

  if (!is.numeric(condi2) || length(condi2) == 0 || is.na(sum(condi2)) ) {
    stop("Input condi2 must be a non-empty non-NA numeric vector.")
  }

  if (!is.matrix(mat1) || !is.numeric(mat1) || is.na(sum(mat1)) ) {
    stop("Input mat1 must be a non-NA numeric matrix.")
  }

  if (!is.matrix(mat2) || !is.numeric(mat2) || is.na(sum(mat2)) ) {
    stop("Input mat2 must be a non-NA numeric matrix.")
  }

  # Check that 2 mats have the same number of columns
  if (ncol(mat1) != ncol(mat2) ) {
    stop("mat1 and mat2 must have the same number of columns.")
  }

  if (ncol(mat1) != length(condi1)) {
    stop("The column of mat1 and the length of vector condi1 must be the same.")
  }

  if (ncol(mat1) != length(condi2)) {
    stop("The column of mat1 and the length of vector condi2 must be the same.")
  }

  cmi_matrix <- do.call(
    rbind,
    lapply(seq_len(nrow(mat1)), function(i) {
      repeated_row <- matrix(
        rep(mat1[i, ], each = nrow(mat2)),
        nrow = nrow(mat2),
        byrow = FALSE
      )

      CMIBiCondimat2mat(
        repeated_row,
        mat2,
        condi1 = condi1,
        condi2 = condi2,
        bin = bin,
        sp_order = sp_order
      )
    })
  )

  rownames(cmi_matrix) <- rownames(mat1)
  colnames(cmi_matrix) <- rownames(mat2)

  return(cmi_matrix)
}

#' @title Calculate Univariate Entropy
#' @description This function calculates the univariate entropy of a numeric vector using the specified
#' number of bins and spline order.
#'
#' @param x_1 A numeric vector for the only variable.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric value representing the entropy of the vector.
#'
#' @examples
#' x_1 <- rnorm(100)
#' getEntropy(x_1)
#'
#' @export
getEntropy <- function(x_1, bin = NULL, sp_order = NULL) {
  # Check inputs

  if (!is.numeric(x_1) || length(x_1) == 0) {
    stop("Input x_1 must be a non-empty numeric vector.")
  }

  # Get the length of x_1
  n <- length(x_1)

  # Check parameters and generate automatically if incorrect
  params <- resolve_params(n, bin, sp_order)

  bin=as.integer(params$bin)
  sp_order=as.integer(params$sp_order)

  # Call the C function for entropy calculation
  out <- .C("compute_entropy_univar",
            x_1 = as.double(x_1),
            n = as.integer(n),
            bin = bin,
            sp_order = sp_order,
            eOut = as.double(0))

  # Extract the entropy value
  e <- out$eOut
  return(e)
}

#' @title Calculate Joint Entropy for Two Variables
#' @description Computes the joint entropy of two numeric vectors using the specified number of bins and spline order.
#'
#' @param x_1 A numeric vector for the first variable.
#' @param x_2 A numeric vector for the second variable. Must be the same length as `x_1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric value representing the joint entropy of the two vectors.
#'
#' @examples
#' x_1 <- rnorm(100)
#' x_2 <- rnorm(100)
#' getEntropyBi(x_1, x_2)
#'
#' @export
getEntropyBi <- function(x_1, x_2, bin = NULL, sp_order = NULL) {
  # Check inputs

  if (!is.numeric(x_1) || length(x_1) == 0) {
    stop("Input x_1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_2) || length(x_2) == 0) {
    stop("Input x_2 must be a non-empty numeric vector.")
  }

  # Check that x_1 and x_2 are of the same length
  if (length(x_1) != length(x_2)) {
    stop("Vectors x_1 and x_2 must be of the same length.")
  }

  # Define the length of the vectors
  n <- length(x_1)

  # Check parameters and generate automatically if incorrect
  params <- resolve_params(n, bin, sp_order)

  bin=as.integer(params$bin)
  sp_order=as.integer(params$sp_order)

  # Call the C function for joint entropy calculation
  out <- .C("compute_entropy_bivar",
            x_1 = as.double(x_1),
            x_2 = as.double(x_2),
            n = as.integer(n),
            bin = bin,
            sp_order = sp_order,
            eOut = as.double(0))

  # Extract the joint entropy value
  joint_entropy <- out$eOut
  return(joint_entropy)
}


#' @title Calculate Joint Entropy for Three Variables
#' @description Computes the joint entropy of three numeric vectors using the specified number of bins and spline order.
#'
#' @param x_1 A numeric vector for the first variable.
#' @param x_2 A numeric vector for the second variable. Must be the same length as `x_1`.
#' @param x_3 A numeric vector for the third variable. Must be the same length as `x_1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric value representing the joint entropy of the three vectors.
#'
#' @examples
#' x_1 <- rnorm(100)
#' x_2 <- rnorm(100)
#' x_3 <- rnorm(100)
#' getEntropyTri(x_1, x_2, x_3)
#'
#' @export
getEntropyTri <- function(x_1, x_2, x_3, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(x_1) || length(x_1) == 0) {
    stop("Input x_1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_2) || length(x_2) == 0) {
    stop("Input x_2 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_3) || length(x_3) == 0) {
    stop("Input x_3 must be a non-empty numeric vector.")
  }

  # Check that x_1, x_2 and x_3 are of the same length
  if (length(x_1) != length(x_2)) {
    stop("Vectors x_1, x_2, and x_3 must be of the same length.")
  }

  if (length(x_2) != length(x_3)) {
    stop("Vectors x_1, x_2, and x_3 must be of the same length.")
  }

  # Get the length of the vectors
  n <- length(x_1)

  # Check parameters and generate automatically if incorrect
  params <- resolve_params(n, bin, sp_order)

  bin=as.integer(params$bin)
  sp_order=as.integer(params$sp_order)


  # Call the C functions to compute the entropy
  out <- .C("compute_entropy_trivar",
            x_1 = as.double(x_1),
            x_2 = as.double(x_2),
            x_3 = as.double(x_3),
            n = as.integer(n),
            bin = bin,
            sp_order = sp_order,
            eOut = as.double(0))

  # Extract the joint entropy value
  joint_entropy <- out$eOut

  return(joint_entropy)
}


#' @title Calculate Joint Entropy for Four Variables
#' @description Computes the joint entropy of four numeric vectors using the specified number of bins and spline order.
#'
#' @param x_1 A numeric vector for the first variable.
#' @param x_2 A numeric vector for the second variable. Must be the same length as `x_1`.
#' @param x_3 A numeric vector for the third variable. Must be the same length as `x_1`.
#' @param x_4 A numeric vector for the fourth variable. Must be the same length as `x_1`.
#' @param bin An integer specifying the number of bins. Default is NULL.
#' @param sp_order An integer specifying the spline order. Must be less than `bin`. Default is NULL.
#' @return A numeric value representing the joint entropy of the four vectors.
#'
#' @examples
#' x_1 <- rnorm(100)
#' x_2 <- rnorm(100)
#' x_3 <- rnorm(100)
#' x_4 <- rnorm(100)
#' getEntropyQuadri(x_1, x_2, x_3, x_4)
#'
#' @export
getEntropyQuadri <- function(x_1, x_2, x_3, x_4, bin = NULL, sp_order = NULL) {
  # Validate inputs as non-empty numeric vectors

  if (!is.numeric(x_1) || length(x_1) == 0) {
    stop("Input x_1 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_2) || length(x_2) == 0) {
    stop("Input x_2 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_3) || length(x_3) == 0) {
    stop("Input x_3 must be a non-empty numeric vector.")
  }

  if (!is.numeric(x_4) || length(x_4) == 0) {
    stop("Input x_4 must be a non-empty numeric vector.")
  }

  # Check that x_1, x_2, x_3, x_4 are of the same length
  if (length(x_1) != length(x_2)) {
    stop("Vectors x_1, x_2, x_3 and x_4 must be of the same length.")
  }

  if (length(x_2) != length(x_3)) {
    stop("Vectors x_1, x_2, x_3 and x_4 must be of the same length.")
  }

  if (length(x_3) != length(x_4)) {
    stop("Vectors x_1, x_2, x_3 and x_4 must be of the same length.")
  }

  # Get the length of the vectors
  n <- length(x_1)

  # Check parameters and generate automatically if incorrect
  params <- resolve_params(n, bin, sp_order)

  bin=as.integer(params$bin)
  sp_order=as.integer(params$sp_order)


  # Call the C functions to compute the entropy
  out <- .C("compute_entropy_quadrivar",
            x_1 = as.double(x_1),
            x_2 = as.double(x_2),
            x_3 = as.double(x_3),
            x_4 = as.double(x_4),
            n = as.integer(n),
            bin = bin,
            sp_order = sp_order,
            eOut = as.double(0))

  # Extract the joint entropy value
  joint_entropy <- out$eOut

  return(joint_entropy)
}

#' @title Internal Utility Function for Parameter Selection
#' @description Resolves and validates the \code{bin} and \code{sp_order} parameters.
#' If \code{bin} is not provided or valid, both \code{bin} and \code{sp_order} are generated automatically.
#' If \code{bin} is valid but \code{sp_order} is not provided or valid, only \code{sp_order} is generated.
#' This function is intended for internal use only and should not be accessed by package users.
#' @keywords internal
#'
#' @param n Integer. Length of the input vector.
#' @param bin Number or \code{NULL}. Number of bins.
#' @param sp_order Number or \code{NULL}. Spline order.
#'
#' @return A list containing the validated or automatically generated values
#' \code{bin} and \code{sp_order}.
resolve_params <- function(n, bin = NULL, sp_order = NULL) {
  if (!is.numeric(n) || length(n) != 1 || is.na(n) || n <= 0) {
    stop("Parameter 'n' must be a positive integer.")
  }

  n <- as.integer(n)

  #bin <- as.integer(bin)
  #sp_order <- as.integer(sp_order)

  if (length(bin)!=1 || is.null(bin) || bin <= 0 || is.na(bin)) {
    bin <- max(4L, round(n/10))
    bin <- min(bin, 12)
    sp_order <- round(bin/6+bin/8)
  } else {
    bin <- as.integer(bin)

    if (length(sp_order)!=1 || is.null(sp_order) || sp_order <= 0 || is.na(sp_order) ) {
      sp_order <- round(bin/6+bin/8)
    } else {
      sp_order <- as.integer(sp_order)
    }
  }

  if (sp_order >= bin) {
    stop("Spline order 'sp_order' must be less than 'bin'.")
  }

  list(bin = bin, sp_order = sp_order)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPvalue.R
\name{getPValue}
\alias{getPValue}
\title{Calculate P-Value from a Sorted Distribution}
\usage{
getPValue(x, sorted_Distri)
}
\arguments{
\item{x}{A numeric value for which the P-value is to be calculated.}

\item{sorted_Distri}{A numeric vector representing a sorted distribution. This distribution must be sorted in ascending order.}
}
\value{
A numeric value indicating the P-value, representing the proportion of values in `sorted_Distri`
that are greater than or equal to `x`.
}
\description{
Computes the P-value for a given numeric value `x` based on its position within a sorted distribution.
This function utilizes a binary search approach for efficient computation.
}
\examples{
x <- rnorm(1)
sorted_dist <- sort(rnorm(100))
getPValue(x, sorted_dist)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeMutualInformation.R
\name{getMI}
\alias{getMI}
\title{Calculate Mutual Information Between Two Vectors}
\usage{
getMI(x_1, x_2, bin = NULL, sp_order = NULL)
}
\arguments{
\item{x_1}{A numeric vector representing the first variable.}

\item{x_2}{A numeric vector representing the second variable. Must be the same length as `x_1`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A numeric value representing the mutual information (MI).
}
\description{
Computes the mutual information (MI) between two numeric vectors
using the specified number of bins and spline order.
}
\examples{
x_1 <- rnorm(100)
x_2 <- rnorm(100)
getMI(x_1, x_2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputePermutation.R
\name{CORmat2vecPermu}
\alias{CORmat2vecPermu}
\title{Permuted Correlation Between Matrix and Vector}
\usage{
CORmat2vecPermu(
  mat,
  vec,
  cor_type = "pearson",
  bulkIdx = 0,
  permutationTimes = 1000,
  seedNum = 99999999
)
}
\arguments{
\item{mat}{A numeric matrix. For example, each row represents a gene and each column represents a sample.}

\item{vec}{A numeric vector, with length equal to the number of columns in `mat`.}

\item{cor_type}{Type of correlation to calculate: "Pearson", "Kendall", or "Spearman". Default is "Pearson".}

\item{bulkIdx}{Index to divide the task when processing many permutations. Default is 0.}

\item{permutationTimes}{Number of permutations for sampling. Default is 1000.}

\item{seedNum}{Seed for random number generation. Default is 99999999.}
}
\value{
A numeric vector of correlation values for each permutation.
}
\description{
Computes the correlation between a randomly sampled vector from a matrix and a given vector.
The sampling is done multiple times to generate a distribution.
}
\examples{
mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
vec <- rnorm(100)
CORmat2vecPermu(mat, vec)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputePermutation.R
\name{additionalRunIF}
\alias{additionalRunIF}
\title{Internal Utility Function for Random Sampling Loops}
\usage{
additionalRunIF(x, y, z, p)
}
\value{
No return value. This function is designed to allow large permutation tests distributed to multiple nodes.
}
\description{
Executes nested loops to perform random sampling operations.
This function is intended for internal use only and should not be accessed by package users.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeMatrix.R
\name{CMIBiCondimat2matAll}
\alias{CMIBiCondimat2matAll}
\title{Normalized Conditional Mutual Information Between Each Row of Two Matrices Given Two Conditions}
\usage{
CMIBiCondimat2matAll(mat1, mat2, condi1, condi2, bin = NULL, sp_order = NULL)
}
\arguments{
\item{mat1}{A numeric matrix. For example, each row represents a gene and each column represents a sample.}

\item{mat2}{Another numeric matrix to compare against. Must have the same columns as `mat1`.}

\item{condi1}{A numeric condition vector, matching the number of columns in `mat1`.}

\item{condi2}{Another numeric condition vector, matching the number of columns in `mat1`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A matrix where each element corresponds to the normalized conditional mutual information (CMI)
between each row of `mat1` and each row of `mat2`, conditioned on `condi1` and `condi2`.
}
\description{
Computes the normalized conditional mutual information (CMI) between each row of first matrix and each row of second matrix,
given two condition variables, normalized by their individual information content. CMI is calculated using
the specified number of bins and spline order.
}
\examples{
mat1 <- matrix(rnorm(1000), nrow = 10, ncol = 100)
mat2 <- matrix(rnorm(20000), nrow = 200, ncol = 100)
condi1 <- rnorm(100)
condi2 <- rnorm(100)
CMIBiCondimat2matAll(mat1, mat2, condi1, condi2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeEntropy.R
\name{getEntropy}
\alias{getEntropy}
\title{Calculate Univariate Entropy}
\usage{
getEntropy(x_1, bin = NULL, sp_order = NULL)
}
\arguments{
\item{x_1}{A numeric vector for the only variable.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A numeric value representing the entropy of the vector.
}
\description{
This function calculates the univariate entropy of a numeric vector using the specified
number of bins and spline order.
}
\examples{
x_1 <- rnorm(100)
getEntropy(x_1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_params.R
\name{resolve_params}
\alias{resolve_params}
\title{Internal Utility Function for Parameter Selection}
\usage{
resolve_params(n, bin = NULL, sp_order = NULL)
}
\arguments{
\item{n}{Integer. Length of the input vector.}

\item{bin}{Number or \code{NULL}. Number of bins.}

\item{sp_order}{Number or \code{NULL}. Spline order.}
}
\value{
A list containing the validated or automatically generated values
\code{bin} and \code{sp_order}.
}
\description{
Resolves and validates the \code{bin} and \code{sp_order} parameters.
If \code{bin} is not provided or valid, both \code{bin} and \code{sp_order} are generated automatically.
If \code{bin} is valid but \code{sp_order} is not provided or valid, only \code{sp_order} is generated.
This function is intended for internal use only and should not be accessed by package users.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputePermutation.R
\name{CMIBiCondimat2matPermu}
\alias{CMIBiCondimat2matPermu}
\title{Permuted Conditional Mutual Information Between Two Matrices Given Two Conditions}
\usage{
CMIBiCondimat2matPermu(
  mat1,
  mat2,
  condi1,
  condi2,
  bin = NULL,
  sp_order = NULL,
  bulkIdx = 0,
  permutationTimes = 1000,
  seedNum = 99999999
)
}
\arguments{
\item{mat1}{A numeric matrix. For example, each row represents a gene and each column represents a sample.}

\item{mat2}{Another numeric matrix to compare against. Must have the same dimensions as `mat1`.}

\item{condi1}{A numeric condition vector, matching the number of columns in `mat1`.}

\item{condi2}{Another numeric condition vector, matching the number of columns in `mat`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}

\item{bulkIdx}{Index to divide the task when processing many permutations. Default is 0.}

\item{permutationTimes}{Number of permutations for sampling. Default is 1000.}

\item{seedNum}{Seed for random number generation. Default is 99999999.}
}
\value{
A numeric vector of normalized conditional mutual information (CMI) values for each permutation.
}
\description{
Computes the normalized conditional mutual information (CMI) between vectors sampled from two matrices,
conditioned on two vectors, normalized by the individual information content.
The sampling is done multiple times to generate a distribution.
}
\examples{
mat1 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
mat2 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
condi1 <- rnorm(100)
condi2 <- rnorm(100)
CMIBiCondimat2matPermu(mat1, mat2, condi1, condi2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeMatrix.R
\name{MImat2matAll}
\alias{MImat2matAll}
\title{Normalized Mutual Information Between Each Row of Two Matrices}
\usage{
MImat2matAll(mat1, mat2, bin = NULL, sp_order = NULL)
}
\arguments{
\item{mat1}{A numeric matrix. For example, each row represents a gene and each column represents a sample.}

\item{mat2}{Another numeric matrix to compare against. Must have the columns as `mat1`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A matrix where each element corresponds to the normalized mutual information (MI) between
respective rows of `mat1` and `mat2`.
}
\description{
Computes the normalized mutual information (MI) between each row of first matrix and each row of second matrix,
normalized by their individual information content, using the specified number of bins and spline order.
}
\examples{
mat1 <- matrix(rnorm(1000), nrow = 10, ncol = 100)
mat2 <- matrix(rnorm(2000), nrow = 200, ncol = 100)
MImat2matAll(mat1, mat2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeMatrix.R
\name{CMImat2matAll}
\alias{CMImat2matAll}
\title{Normalized Conditional Mutual Information Between Each Row of Two Matrices}
\usage{
CMImat2matAll(mat1, mat2, condi, bin = NULL, sp_order = NULL)
}
\arguments{
\item{mat1}{A numeric matrix. For example, each row represents a gene and each column represents a sample.}

\item{mat2}{Another numeric matrix to compare against. Must have the same columns as `mat1`.}

\item{condi}{A numeric condition vector, matching the number of columns in `mat1`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A matrix where each element corresponds to the normalized conditional mutual information (CMI)
between each row of `mat1` and each row of `mat2`, conditioned on `condi`.
}
\description{
Computes the normalized conditional mutual information (CMI) between each row of first matrix and each row of second matrix,
given a condition variable, normalized by their individual information content. CMI is calculated using the
specified number of bins and spline order.
}
\examples{
mat1 <- matrix(rnorm(1000), nrow = 10, ncol = 100)
mat2 <- matrix(rnorm(20000), nrow = 200, ncol = 100)
condi <- rnorm(100)
CMImat2matAll(mat1, mat2, condi)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputePermutation.R
\name{MImat2vecPermu}
\alias{MImat2vecPermu}
\title{Permuted Normalized Mutual Information Between Matrix and Vector}
\usage{
MImat2vecPermu(
  mat,
  vec,
  bin = NULL,
  sp_order = NULL,
  bulkIdx = 0,
  permutationTimes = 1000,
  seedNum = 99999999
)
}
\arguments{
\item{mat}{A numeric matrix. For example, each row represents a gene and each column represents a sample.}

\item{vec}{A numeric vector, with length equal to the number of columns in `mat`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}

\item{bulkIdx}{Index to divide the task when processing many permutations. Default is 0.}

\item{permutationTimes}{Number of permutations for sampling. Default is 1000.}

\item{seedNum}{Seed for random number generation. Default is 99999999.}
}
\value{
A numeric vector of normalized mutual information (MI) values for each permutation.
}
\description{
Computes the mutual information (MI) between a random vector sampled from a matrix and a vector,
normalized by the mutual information of the vector with itself. The sampling is done multiple times to generate a distribution.
}
\examples{
mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
vec <- rnorm(100)
MImat2vecPermu(mat, vec)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeEntropy.R
\name{getEntropyQuadri}
\alias{getEntropyQuadri}
\title{Calculate Joint Entropy for Four Variables}
\usage{
getEntropyQuadri(x_1, x_2, x_3, x_4, bin = NULL, sp_order = NULL)
}
\arguments{
\item{x_1}{A numeric vector for the first variable.}

\item{x_2}{A numeric vector for the second variable. Must be the same length as `x_1`.}

\item{x_3}{A numeric vector for the third variable. Must be the same length as `x_1`.}

\item{x_4}{A numeric vector for the fourth variable. Must be the same length as `x_1`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A numeric value representing the joint entropy of the four vectors.
}
\description{
Computes the joint entropy of four numeric vectors using the specified number of bins and spline order.
}
\examples{
x_1 <- rnorm(100)
x_2 <- rnorm(100)
x_3 <- rnorm(100)
x_4 <- rnorm(100)
getEntropyQuadri(x_1, x_2, x_3, x_4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeMatrix.R
\name{MImat2mat}
\alias{MImat2mat}
\title{Normalized Mutual Information Between Two Matrices}
\usage{
MImat2mat(mat1, mat2, bin = NULL, sp_order = NULL)
}
\arguments{
\item{mat1}{A numeric matrix. For example, each row represents a gene and each column represents a sample.}

\item{mat2}{Another numeric matrix to compare against. Must have the same dimensions as `mat1`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A numeric vector where each element corresponds to the normalized mutual information (MI) between
respective rows of `mat1` and `mat2`.
}
\description{
Computes the normalized mutual information (MI) between the same rows of two matrices
normalized by their individual information content, using the specified number of bins and spline order.
}
\examples{
mat1 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
mat2 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
MImat2mat(mat1, mat2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeMatrix.R
\name{CMIBiCondimat2mat}
\alias{CMIBiCondimat2mat}
\title{Normalized Conditional Mutual Information Between Two Matrices Given Two Conditions}
\usage{
CMIBiCondimat2mat(mat1, mat2, condi1, condi2, bin = NULL, sp_order = NULL)
}
\arguments{
\item{mat1}{A numeric matrix. For example, each row represents a gene and each column represents a sample.}

\item{mat2}{Another numeric matrix to compare against. Must have the same dimensions as `mat1`.}

\item{condi1}{A numeric condition vector, matching the number of columns in `mat1`.}

\item{condi2}{Another numeric condition vector, matching the number of columns in `mat1`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A numeric vector representing the normalized conditional mutual information (CMI) between pairs of rows
from `mat1` and `mat2`, conditioned on `condi1` and `condi2`.
}
\description{
Computes the normalized conditional mutual information (CMI) between the same rows of two matrices,
given two condition variables, normalized by their individual information content. CMI is calculated using the specified number of bins and spline order.
}
\examples{
mat1 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
mat2 <- matrix(rnorm(10000), nrow = 100, ncol = 100)
condi1 <- rnorm(100)
condi2 <- rnorm(100)
CMIBiCondimat2mat(mat1, mat2, condi1, condi2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeMatrix.R
\name{CMIBiCondimat2vec}
\alias{CMIBiCondimat2vec}
\title{Normalized Conditional Mutual Information Between Matrix and Vector Given Two Conditions}
\usage{
CMIBiCondimat2vec(mat, vec, condi1, condi2, bin = NULL, sp_order = NULL)
}
\arguments{
\item{mat}{A numeric matrix. For example, each row represents a gene and each column represents a sample.}

\item{vec}{A numeric vector, with length equal to the number of columns in `mat`.}

\item{condi1}{A numeric condition vector, matching the number of columns in `mat`.}

\item{condi2}{Another numeric condition vector, matching the number of columns in `mat`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A numeric vector representing the normalized conditional mutual information (CMI) between each row of `mat` and `vec`, given `condi1` and `condi2`.
}
\description{
Computes the normalized conditional mutual information (CMI) between each row of a matrix and a vector,
given two condition vectors, normalized by the mutual information of the vector with itself using the specified bins and spline order.
}
\examples{
mat <- matrix(rnorm(10000), nrow = 100, ncol = 100)
vec <- rnorm(100)
condi1 <- rnorm(100)
condi2 <- rnorm(100)
CMIBiCondimat2vec(mat, vec, condi1, condi2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeMutualInformation.R
\name{getMIBi}
\alias{getMIBi}
\title{Calculate Joint Mutual Information}
\usage{
getMIBi(x_1, x_2, x_3, bin = NULL, sp_order = NULL)
}
\arguments{
\item{x_1}{A numeric vector for the first variable.}

\item{x_2}{A numeric vector for the second variable. Must match the length of `x_1`.}

\item{x_3}{A numeric vector for the third variable. Must match the length of `x_1`.}

\item{bin}{An integer specifying the number of bins. Default is NULL.}

\item{sp_order}{An integer specifying the spline order. Must be less than `bin`. Default is NULL.}
}
\value{
A numeric value representing joint mutual information (MI).
}
\description{
Computes the joint mutual information I(x_1, x_2; x_3)
using the specified number of bins and spline order.
}
\examples{
x_1 <- rnorm(100)
x_2 <- rnorm(100)
x_3 <- rnorm(100)
getMIBi(x_1, x_2, x_3)

}
